@@ -1,11 +1,11 @@
 """Kazoo Zookeeper Client"""
-import inspect
-import logging
-import re
-import warnings
 from collections import defaultdict, deque
 from functools import partial
+import inspect
+import logging
 from os.path import split
+import re
+import warnings
 
 import six
 
@@ -25,8 +25,7 @@ from kazoo.handlers.utils import capture_exceptions, wrap
 from kazoo.hosts import collect_hosts
 from kazoo.loggingsupport import BLATHER
 from kazoo.protocol.connection import ConnectionHandler
-from kazoo.protocol.paths import normpath
-from kazoo.protocol.paths import _prefix_root
+from kazoo.protocol.paths import _prefix_root, normpath
 from kazoo.protocol.serialization import (
     Auth,
     CheckVersion,
@@ -44,33 +43,27 @@ from kazoo.protocol.serialization import (
     Sync,
     Transaction
 )
-from kazoo.protocol.states import Callback
-from kazoo.protocol.states import EventType
-from kazoo.protocol.states import KazooState
-from kazoo.protocol.states import KeeperState
-from kazoo.protocol.states import WatchedEvent
+from kazoo.protocol.states import (
+    Callback,
+    EventType,
+    KazooState,
+    KeeperState,
+    WatchedEvent
+)
 from kazoo.retry import KazooRetry
-from kazoo.security import ACL
-from kazoo.security import OPEN_ACL_UNSAFE
+from kazoo.security import ACL, OPEN_ACL_UNSAFE
 
 # convenience API
-from kazoo.recipe.barrier import Barrier
-from kazoo.recipe.barrier import DoubleBarrier
+from kazoo.recipe.barrier import Barrier, DoubleBarrier
 from kazoo.recipe.counter import Counter
 from kazoo.recipe.election import Election
-from kazoo.recipe.lease import NonBlockingLease
-from kazoo.recipe.lease import MultiNonBlockingLease
-from kazoo.recipe.lock import Lock
-from kazoo.recipe.lock import ReadLock
-from kazoo.recipe.lock import WriteLock
-from kazoo.recipe.lock import Semaphore
+from kazoo.recipe.lease import NonBlockingLease, MultiNonBlockingLease
+from kazoo.recipe.lock import Lock, ReadLock, WriteLock, Semaphore
 from kazoo.recipe.partitioner import SetPartitioner
-from kazoo.recipe.party import Party
-from kazoo.recipe.party import ShallowParty
-from kazoo.recipe.queue import Queue
-from kazoo.recipe.queue import LockingQueue
-from kazoo.recipe.watchers import ChildrenWatch
-from kazoo.recipe.watchers import DataWatch
+from kazoo.recipe.party import Party, ShallowParty
+from kazoo.recipe.queue import Queue, LockingQueue
+from kazoo.recipe.watchers import ChildrenWatch, DataWatch
+
 
 string_types = six.string_types
 bytes_types = (six.binary_type,)
@@ -86,7 +79,6 @@ _RETRY_COMPAT_DEFAULTS = dict(
     max_retries=None,
     retry_delay=0.1,
     retry_backoff=2,
-    retry_jitter=0.8,
     retry_max_delay=3600,
 )
 
@@ -94,7 +86,6 @@ _RETRY_COMPAT_MAPPING = dict(
     max_retries='max_tries',
     retry_delay='delay',
     retry_backoff='backoff',
-    retry_jitter='max_jitter',
     retry_max_delay='max_delay',
 )
 
@@ -113,7 +104,9 @@ class KazooClient(object):
                  timeout=10.0, client_id=None, handler=None,
                  default_acl=None, auth_data=None, read_only=None,
                  randomize_hosts=True, connection_retry=None,
-                 command_retry=None, logger=None, **kwargs):
+                 command_retry=None, logger=None, keyfile=None,
+                 keyfile_password=None, certfile=None, ca=None,
+                 use_ssl=False, verify_certs=True, **kwargs):
         """Create a :class:`KazooClient` instance. All time arguments
         are in seconds.
 
@@ -142,6 +135,13 @@ class KazooClient(object):
             options which will be used for creating one.
         :param logger: A custom logger to use instead of the module
             global `log` instance.
+        :param keyfile: SSL keyfile to use for authentication
+        :param keyfile_password: SSL keyfile password
+        :param certfile: SSL certfile to use for authentication
+        :param ca: SSL CA file to use for authentication
+        :param use_ssl: argument to control whether SSL is used or not
+        :param verify_certs: when using SSL, argument to bypass
+            certs verification
 
         Basic Example:
 
@@ -190,6 +190,12 @@ class KazooClient(object):
         self.chroot = None
         self.set_hosts(hosts)
 
+        self.use_ssl = use_ssl
+        self.verify_certs = verify_certs
+        self.certfile = certfile
+        self.keyfile = keyfile
+        self.keyfile_password = keyfile_password
+        self.ca = ca
         # Curator like simplified state tracking, and listeners for
         # state transitions
         self._state = KeeperState.CLOSED
@@ -297,6 +303,15 @@ class KazooClient(object):
         self.Semaphore = partial(Semaphore, self)
         self.ShallowParty = partial(ShallowParty, self)
 
+        # Managing SASL client
+        self.use_sasl = False
+        for scheme, auth in self.auth_data:
+            if scheme == "sasl":
+                self.use_sasl = True
+                # Could be used later for GSSAPI implementation
+                self.sasl_server_principal = "zk-sasl-md5"
+                break
+
         # If we got any unhandled keywords, complain like Python would
         if kwargs:
             raise TypeError('__init__() got unexpected keyword arguments: %s'
@@ -655,7 +670,14 @@ class KazooClient(object):
 
         peer = self._connection._socket.getpeername()[:2]
         sock = self.handler.create_connection(
-            peer, timeout=self._session_timeout / 1000.0)
+            peer, timeout=self._session_timeout / 1000.0,
+            use_ssl=self.use_ssl,
+            ca=self.ca,
+            certfile=self.certfile,
+            keyfile=self.keyfile,
+            keyfile_password=self.keyfile_password,
+            verify_certs=self.verify_certs,
+        )
         sock.sendall(cmd)
         result = sock.recv(8192)
         sock.close()
@@ -715,8 +737,12 @@ class KazooClient(object):
         """Send credentials to server.
 
         :param scheme: authentication scheme (default supported:
-                       "digest").
+                       "digest", "sasl"). Note that "sasl" scheme is
+                       requiring "pure-sasl" library to be
+                       installed.
         :param credential: the credential -- value depends on scheme.
+                           "digest": user:password
+                           "sasl": user:password
 
         :returns: True if it was successful.
         :rtype: bool
@@ -1004,7 +1030,7 @@ class KazooClient(object):
             returns a non-zero error code.
 
         """
-        return self.exists_async(path, watch).get()
+        return self.exists_async(path, watch=watch).get()
 
     def exists_async(self, path, watch=None):
         """Asynchronously check if a node exists. Takes the same
@@ -1046,7 +1072,7 @@ class KazooClient(object):
             returns a non-zero error code
 
         """
-        return self.get_async(path, watch).get()
+        return self.get_async(path, watch=watch).get()
 
     def get_async(self, path, watch=None):
         """Asynchronously get the value of a node. Takes the same
@@ -1099,7 +1125,8 @@ class KazooClient(object):
             The `include_data` option.
 
         """
-        return self.get_children_async(path, watch, include_data).get()
+        return self.get_children_async(path, watch=watch,
+                                       include_data=include_data).get()
 
     def get_children_async(self, path, watch=None, include_data=False):
         """Asynchronously get a list of child nodes of a path. Takes
@@ -1197,7 +1224,7 @@ class KazooClient(object):
             raise TypeError("Invalid type for 'path' (string expected)")
         if isinstance(acls, ACL) or not isinstance(acls, (tuple, list)):
             raise TypeError("Invalid type for 'acl' (acl must be a tuple/list"
-                            " of ACL's")
+                            " of ACL's)")
         if not isinstance(version, int):
             raise TypeError("Invalid type for 'version' (int expected)")
 
