@@ -1,13 +1,13 @@
 """Zookeeper Protocol Connection Handler"""
+from binascii import hexlify
+from contextlib import contextmanager
+import copy
 import logging
 import random
 import select
 import socket
-
 import sys
 import time
-from binascii import hexlify
-from contextlib import contextmanager
 
 from kazoo.exceptions import (
     AuthFailedError,
@@ -23,9 +23,11 @@ from kazoo.protocol.serialization import (
     Connect,
     Exists,
     GetChildren,
+    GetChildren2,
     Ping,
     PingInstance,
     ReplyHeader,
+    SASL,
     Transaction,
     Watch,
     int_struct
@@ -40,6 +42,12 @@ from kazoo.retry import (
     ForceRetryError,
     RetryFailedError
 )
+try:
+    from puresasl.client import SASLClient
+    PURESASL_AVAILABLE = True
+except ImportError:
+    PURESASL_AVAILABLE = False
+
 
 log = logging.getLogger(__name__)
 
@@ -151,9 +159,12 @@ class ConnectionHandler(object):
         self._xid = None
         self._rw_server = None
         self._ro_mode = False
+        self._ro = False
 
         self._connection_routine = None
 
+        self.sasl_cli = None
+
     # This is instance specific to avoid odd thread bug issues in Python
     # during shutdown global cleanup
     @contextmanager
@@ -216,13 +227,21 @@ class ConnectionHandler(object):
         remaining = length
         with self._socket_error_handling():
             while remaining > 0:
-                s = self.handler.select([self._socket], [], [], timeout)[0]
-                if not s:  # pragma: nocover
-                    # If the read list is empty, we got a timeout. We don't
-                    # have to check wlist and xlist as we don't set any
-                    raise self.handler.timeout_exception("socket time-out"
-                                                         " during read")
-
+                # Because of SSL framing, a select may not return when using
+                # an SSL socket because the underlying physical socket may not
+                # have anything to select, but the wrapped object may still
+                # have something to read as it has previously gotten enough
+                # data from the underlying socket.
+                if (hasattr(self._socket, "pending")
+                        and self._socket.pending() > 0):
+                    pass
+                else:
+                    s = self.handler.select([self._socket], [], [], timeout)[0]
+                    if not s:  # pragma: nocover
+                        # If the read list is empty, we got a timeout. We don't
+                        # have to check wlist and xlist as we don't set any
+                        raise self.handler.timeout_exception(
+                            "socket time-out during read")
                 chunk = self._socket.recv(remaining)
                 if chunk == b'':
                     raise ConnectionDropped('socket connection broken')
@@ -380,7 +399,7 @@ class ConnectionHandler(object):
             # Determine if watchers should be registered
             watcher = getattr(request, 'watcher', None)
             if not client._stopped.is_set() and watcher:
-                if isinstance(request, GetChildren):
+                if isinstance(request, (GetChildren, GetChildren2)):
                     client._child_watchers[request.path].add(watcher)
                 else:
                     client._data_watchers[request.path].add(watcher)
@@ -408,6 +427,24 @@ class ConnectionHandler(object):
                 async_object.set(True)
         elif header.xid == WATCH_XID:
             self._read_watch_event(buffer, offset)
+        elif self.sasl_cli and not self.sasl_cli.complete:
+            # SASL authentication is not yet finished, this can only
+            # be a SASL packet
+            self.logger.log(BLATHER, 'Received SASL')
+            try:
+                challenge, _ = SASL.deserialize(buffer, offset)
+            except Exception:
+                raise ConnectionDropped('error while SASL authentication.')
+            response = self.sasl_cli.process(challenge)
+            if response:
+                # authentication not yet finished, answering the challenge
+                self._send_sasl_request(challenge=response,
+                                        timeout=client._session_timeout)
+            else:
+                # authentication is ok, state is CONNECTED or CONNECTED_RO
+                # remove sensible information from the object
+                self._set_connected_ro_or_rw(client)
+                self.sasl_cli.dispose()
         else:
             self.logger.log(BLATHER, 'Reading for header %r', header)
 
@@ -536,11 +573,11 @@ class ConnectionHandler(object):
             client._session_callback(KeeperState.CONNECTING)
 
         try:
+            self._xid = 0
             read_timeout, connect_timeout = self._connect(host, port)
             read_timeout = read_timeout / 1000.0
             connect_timeout = connect_timeout / 1000.0
             retry.reset()
-            self._xid = 0
             self.ping_outstanding.clear()
             with self._socket_error_handling():
                 while not close_connection:
@@ -596,7 +633,8 @@ class ConnectionHandler(object):
 
     def _connect(self, host, port):
         client = self.client
-        self.logger.info('Connecting to %s:%s', host, port)
+        self.logger.info('Connecting to %s:%s, use_ssl: %r',
+                         host, port, self.client.use_ssl)
 
         self.logger.log(BLATHER,
                         '    Using session_id: %r session_passwd: %s',
@@ -605,7 +643,15 @@ class ConnectionHandler(object):
 
         with self._socket_error_handling():
             self._socket = self.handler.create_connection(
-                (host, port), client._session_timeout / 1000.0)
+                address=(host, port),
+                timeout=client._session_timeout / 1000.0,
+                use_ssl=self.client.use_ssl,
+                keyfile=self.client.keyfile,
+                certfile=self.client.certfile,
+                ca=self.client.ca,
+                keyfile_password=self.client.keyfile_password,
+                verify_certs=self.client.verify_certs,
+            )
 
         self._socket.setblocking(0)
 
@@ -614,7 +660,7 @@ class ConnectionHandler(object):
                           client.read_only)
 
         connect_result, zxid = self._invoke(
-            client._session_timeout / 1000.0, connect)
+            client._session_timeout / 1000.0 / len(client.hosts), connect)
 
         if connect_result.time_out <= 0:
             raise SessionExpiredError("Session has expired")
@@ -640,16 +686,68 @@ class ConnectionHandler(object):
                         read_timeout)
 
         if connect_result.read_only:
+            self._ro = True
+
+        # Get a copy of the auth data before iterating, in case it is
+        # changed.
+        client_auth_data_copy = copy.copy(client.auth_data)
+
+        if client.use_sasl and self.sasl_cli is None:
+            if PURESASL_AVAILABLE:
+                for scheme, auth in client_auth_data_copy:
+                    if scheme == 'sasl':
+                        username, password = auth.split(":")
+                        self.sasl_cli = SASLClient(
+                            host=client.sasl_server_principal,
+                            service='zookeeper',
+                            mechanism='DIGEST-MD5',
+                            username=username,
+                            password=password
+                        )
+                        break
+
+                # As described in rfc
+                # https://tools.ietf.org/html/rfc2831#section-2.1
+                # sending empty challenge
+                self._send_sasl_request(challenge=b'',
+                                        timeout=connect_timeout)
+            else:
+                self.logger.warn('Pure-sasl library is missing while sasl'
+                                 ' authentification is configured. Please'
+                                 ' install pure-sasl library to connect '
+                                 'using sasl. Now falling back '
+                                 'connecting WITHOUT any '
+                                 'authentification.')
+                client.use_sasl = False
+                self._set_connected_ro_or_rw(client)
+        else:
+            self._set_connected_ro_or_rw(client)
+            for scheme, auth in client_auth_data_copy:
+                if scheme == "digest":
+                    ap = Auth(0, scheme, auth)
+                    zxid = self._invoke(
+                        connect_timeout / 1000.0,
+                        ap,
+                        xid=AUTH_XID
+                    )
+                    if zxid:
+                        client.last_zxid = zxid
+
+        return read_timeout, connect_timeout
+
+    def _send_sasl_request(self, challenge, timeout):
+        """ Called when sending a SASL request, xid needs be to incremented """
+        sasl_request = SASL(challenge)
+        self._xid = (self._xid % 2147483647) + 1
+        xid = self._xid
+        self._submit(sasl_request, timeout / 1000.0, xid)
+
+    def _set_connected_ro_or_rw(self, client):
+        """ Called to decide whether to set the KeeperState to CONNECTED_RO
+            or CONNECTED"""
+        if self._ro:
             client._session_callback(KeeperState.CONNECTED_RO)
             self._ro_mode = iter(self._server_pinger())
         else:
             client._session_callback(KeeperState.CONNECTED)
             self._ro_mode = None
-
-        for scheme, auth in client.auth_data:
-            ap = Auth(0, scheme, auth)
-            zxid = self._invoke(connect_timeout / 1000.0, ap, xid=AUTH_XID)
-            if zxid:
-                client.last_zxid = zxid
-
-        return read_timeout, connect_timeout
