@@ -2,12 +2,13 @@
 from collections import namedtuple
 import struct
 
+import six
+
 from kazoo.exceptions import EXCEPTIONS
 from kazoo.protocol.states import ZnodeStat
 from kazoo.security import ACL
 from kazoo.security import Id
 
-import six
 
 # Struct objects with formats compiled
 bool_struct = struct.Struct('B')
@@ -377,6 +378,19 @@ class Auth(namedtuple('Auth', 'auth_type scheme auth')):
                 write_string(self.auth))
 
 
+class SASL(namedtuple('SASL', 'challenge')):
+    type = 102
+
+    def serialize(self):
+        b = bytearray()
+        b.extend(write_buffer(self.challenge))
+        return b
+
+    @classmethod
+    def deserialize(cls, bytes, offset):
+        challenge, offset = read_buffer(bytes, offset)
+        return challenge, offset
+
 class Watch(namedtuple('Watch', 'type state path')):
     @classmethod
     def deserialize(cls, bytes, offset):
