@@ -179,6 +179,34 @@ class TestAuthentication(KazooTestCase):
             client.stop()
             client.close()
 
+    def test_connect_sasl_auth(self):
+        from kazoo.security import make_acl
+
+        if TRAVIS_ZK_VERSION:
+            version = TRAVIS_ZK_VERSION
+        else:
+            version = self.client.server_version()
+        if not version or version < (3, 4):
+            raise SkipTest("Must use Zookeeper 3.4 or above")
+
+        username = "jaasuser"
+        password = "jaas_password"
+        sasl_auth = "%s:%s" % (username, password)
+
+        acl = make_acl('sasl', credential=username, all=True)
+
+        client = self._get_client(auth_data=[('sasl', sasl_auth)])
+        client.start()
+        try:
+            client.create('/1', acl=(acl,))
+            # give ZK a chance to copy data to other node
+            time.sleep(0.1)
+            self.assertRaises(NoAuthError, self.client.get, "/1")
+        finally:
+            client.delete('/1')
+            client.stop()
+            client.close()
+
     def test_unicode_auth(self):
         username = u("xe4/\hm")
         password = u("/\xe4hm")
@@ -217,6 +245,16 @@ class TestAuthentication(KazooTestCase):
         self.assertRaises(TypeError, client.add_auth,
                           None, ('user', 'pass'))
 
+    def test_invalid_sasl_auth(self):
+        if TRAVIS_ZK_VERSION:
+            version = TRAVIS_ZK_VERSION
+        else:
+            version = self.client.server_version()
+        if not version or version < (3, 4):
+            raise SkipTest("Must use Zookeeper 3.4 or above")
+        client = self._get_client(auth_data=[('sasl', 'baduser:badpassword')])
+        self.assertRaises(ConnectionLoss, client.start)
+
     def test_async_auth(self):
         client = self._get_client()
         client.start()
@@ -1116,7 +1154,7 @@ class TestClientTransactions(KazooTestCase):
         eq_(self.client.get('/smith')[0], b'32')
 
 
-class TestCallbacks(unittest.TestCase):
+class TestSessionCallbacks(unittest.TestCase):
     def test_session_callback_states(self):
         from kazoo.protocol.states import KazooState, KeeperState
         from kazoo.client import KazooClient
@@ -1147,6 +1185,28 @@ class TestCallbacks(unittest.TestCase):
         eq_(client.state, KazooState.SUSPENDED)
 
 
+class TestCallbacks(KazooTestCase):
+    def test_async_result_callbacks_are_always_called(self):
+        # create a callback object
+        callback_mock = mock.Mock()
+
+        # simulate waiting for a response
+        async_result = self.client.handler.async_result()
+        async_result.rawlink(callback_mock)
+
+        # begin the procedure to stop the client
+        self.client.stop()
+
+        # the response has just been received;
+        # this should be on another thread,
+        # simultaneously with the stop procedure
+        async_result.set_exception(
+            Exception("Anything that throws an exception"))
+
+        # with the fix the callback should be called
+        self.assertGreater(callback_mock.call_count, 0)
+
+
 class TestNonChrootClient(KazooTestCase):
 
     def test_create(self):
@@ -1166,9 +1226,9 @@ class TestNonChrootClient(KazooTestCase):
 
 
 class TestReconfig(KazooTestCase):
-
     def setUp(self):
         KazooTestCase.setUp(self)
+
         if TRAVIS_ZK_VERSION:
             version = TRAVIS_ZK_VERSION
         else:
@@ -1176,12 +1236,25 @@ class TestReconfig(KazooTestCase):
         if not version or version < (3, 5):
             raise SkipTest("Must use Zookeeper 3.5 or above")
 
+    def test_no_super_auth(self):
+        self.assertRaises(NoAuthError,
+                          self.client.reconfig,
+                          joining='server.999=0.0.0.0:1234:2345:observer;3456',
+                          leaving=None,
+                          new_members=None)
+
     def test_add_remove_observer(self):
         def free_sock_port():
             s = socket.socket()
             s.bind(('', 0))
             return s, s.getsockname()[1]
 
+        username = "super"
+        password = "test"
+        digest_auth = "%s:%s" % (username, password)
+        client = self._get_client(auth_data=[('digest', digest_auth)])
+        client.start()
+
         # get ports for election, zab and client endpoints. we need to use
         # ports for which we'd immediately get a RST upon connect(); otherwise
         # the cluster could crash if it gets a SocketTimeoutException:
@@ -1192,18 +1265,18 @@ class TestReconfig(KazooTestCase):
 
         joining = 'server.100=0.0.0.0:%d:%d:observer;0.0.0.0:%d' % (
             port1, port2, port3)
-        data, _ = self.client.reconfig(joining=joining,
+        data, _ = client.reconfig(joining=joining,
                                        leaving=None,
                                        new_members=None)
-        self.assertIn(joining, data)
+        self.assertIn(joining.encode('utf8'), data)
 
-        data, _ = self.client.reconfig(joining=None,
+        data, _ = client.reconfig(joining=None,
                                        leaving='100',
                                        new_members=None)
-        self.assertNotIn(joining, data)
+        self.assertNotIn(joining.encode('utf8'), data)
 
         # try to add it again, but a config number in the future
-        curver = int(data.split('\n')[-1].split('=')[1], base=16)
+        curver = int(data.decode().split('\n')[-1].split('=')[1], base=16)
         self.assertRaises(BadVersionError,
                           self.client.reconfig,
                           joining=joining,
